indexing

	description	: "GERL implementation for ERL_CREATION_PROCEDURE"
	library: "Erl-G"
	copyright: "Copyright (c) 2005, Beat Fluri and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

class ERL_CREATION_PROCEDURE_IMP

inherit

	ERL_CREATION_PROCEDURE

create
	
	make
	
feature {NONE} -- Initialization

	make (a_name: like name; 
			a_formal_arguments: like formal_arguments;
			a_type: like type;
			a_precondition_function: like precondition_function;
			a_creation_function: like creation_function) is
			-- Create new creation procedure.
		require
			a_name_not_void: a_name /= Void
			a_formal_arguments_not_void: a_formal_arguments /= Void
			a_formal_arguments_doesnt_have_void: not a_formal_arguments.has (Void)
			a_precondition_function_not_void: a_precondition_function /= Void
			precondition_argument_match: a_precondition_function.operands.count = a_formal_arguments.count 
			a_creation_function_not_void: a_creation_function /= Void
			creation_function_argument_match: a_creation_function.operands.count = a_formal_arguments.count
		do
			name := a_name
			formal_arguments := a_formal_arguments
			precondition_function := a_precondition_function
			creation_function := a_creation_function
		ensure
			name_set: name = a_name
			formal_arguments_set: formal_arguments = a_formal_arguments
			precondition_function_set: precondition_function = a_precondition_function
			creation_function_set: creation_function = a_creation_function	
		end

feature -- Access

	name: STRING
			-- Name of this creation procedure

    formal_arguments: LIST [ERL_ARGUMENT]
            -- Formal arguments of this creation procedure
        
    type: ERL_TYPE
		    -- Type of objects that this creation procedure creates

    last_result: ANY is
		    -- Last object created
		do
			Result := creation_function.last_result
		end

	precondition (a_arguments: TUPLE): BOOLEAN is
			-- Do `a_target' and `a_operands' satisfy creation procedure precondition?
		do
			precondition_function.apply
			Result := precondition_function.last_result
		end

feature -- Status report

    valid_arguments (a_arguments: TUPLE): BOOLEAN is
            -- Are `a_operands' valid operands?
        do
	        Result := creation_function.valid_operands (a_arguments)
        end
    
feature -- Basic operations

	apply (a_arguments: TUPLE)is
			-- Call creation routine with operands `operands'.
		do
			creation_function.call (a_arguments)
		end
		
feature {NONE} -- Implementation

	precondition_function: FUNCTION [ANY, TUPLE, BOOLEAN]
		-- Precondition of creation procedure

	creation_function: FUNCTION [ANY, TUPLE, ANY]
		-- Creation procedure packed into a function agent

invariant

	precondition_function_not_void: precondition_function /= Void
	precondition_argument_match: precondition_function.operands.count = formal_arguments.count
	creation_function_not_oid: creation_function /= Void 
	creation_function_argument_match: creation_function.operands.count = formal_arguments.count

end
